      program omretrieve
c
c     Code to retrieve optical model potentials from the RIPL-2 optical
c     model potential (omp) library and to format them for input into
c     the SCAT2000 and ECIS96 computer codes.
c
c     Version date: 13 March 2003
c     Small changes introduced in the dispersive optical model package
c     to improve accuracy of the volume dispersive correction calculation
c     RCN
c
c     Previous Version date:  5 July 2002
c
c     P.G.Young, Group T-2, Los Alamos National Laboratory
c     Mail Stop B243, email address pgy@lanl.gov
c
c     Coding for dispersive potentials and gamma factor by:
c     R. Capote Noy, Centro de Estudios Aplicados al Desarrollo Nuclear,
c     Cuba, email address rcapotenoy@yahoo.com or rcapote@infomed.sld.cu
c
c     Code compiled using:
c         f77 om-retrieve.f -C -O3 -o omretrieve
c     That is, -r8 should not be turned on in order for the dispersive
c     corrections to be computed accurately.
c
c     INPUT FILES
c       omparameter.dat = RIPL2 Optical Model Potential (OMP) file
c       gs-mass-sp.dat = ground-state mass, spin-parity file
c       ominput.inp = input instructions, defined below
c
c     OUTPUT FILES
c       sc2.inp   = input file for the scat2000 code (modtyp=1)
c       ec96.inp  = standard input file for the ecis96 code (modtyp=2)
c       ec96tc.inp= alternate input file for the ecis96 code.
c                   (modtyp=2).  Useful for generating transmission
c                   transmission coefficients.
c       ec96vib   = input file for the ecis96 code with vibrational
c                   model activated (modtyp=2).
c       ec96dw.inp= input file for the ecis96 code with DWBA
c                   model activated (modtyp=3).
c       massinfo.out = descriptive remarks about the ground-state mass
c                      data used in the gs-mass-sp.dat file.
c       omp-table = a table of numerical values of each potential made
c                   at each incident energy when modtyp=2 (ECIS input).
c
c     INPUT DETAILS (ominput.inp file)
c
c     read(5,*) ne
c
c       ne = number of incident energies
c          = 0 to use a built-in array of incident energies
c
c     if(ne.gt.0) read(5,*) (en(n),n=1,ne)
c
c       en(n) = incident energies in MeV in the laboratory system
c
c     read(5,*,end=990) iztar,iatar,irefget,modtyp
c
c       abs(iztar) = Z of target nucleus
c                    Set iztar negative to provide integer projectile
c                    mass in input decks.
c       abs(iatar) = A of target nucleus
c                    Set iatar negative to provide integer target
c                    mass in input decks.
c
c       irefget = reference number of optical model potential to be
c                 retrieved from the RIPL library
c         Set irefget = negative izaproj (projectile) to retrieve all
c                       spherical potentials in the RIPL library for
c                       this izaproj and the inputted izatar (target).
c
c       iabs(modtyp)= 1 to generate SCAT2 input file (sc2.inp)
c       iabs(modtyp)= 2 to generate ECIS96 input files (ec96.inp,
c                       ec96tc.inp)
c       iabs(modtyp)= 3 to generate ECIS96 DWBA input files
c                       (ec96dw.inp), using structure information from
c                       an external file (deform.dat), which may be the
c                       om-deformations.dat*file or a user-provided
c                       deform.dat file.
c
c     if(iabs(modtyp).eq.3)read(5,*)kdef
c
c       Set modtyp negative to force imaginary volume and surface
c       potentials to be zero or positive.  If iabs(modtyp)=1, then
c       all SCAT2000 inputs are single energy, i.e., the compact
c       energy representation of SCAT2000 is bypassed.
c
c
c         kdef=1 use JENDL-3.2 betas from om-deformations.dat file
c             =2 use ENSDF(Q)
c             =3 use ENSDF(BE2) and ENSDF(BE3)
c             =4 use Raman and Spear for BE2,BE3
c             =5 use deform.dat file provided by user
c
c       See om-deformations.readme file for more information on the
c       kdef options and for the format of deform.dat for using kdef=5.
c       For kdef=1-4, the om-deformations.dat file must be copied to
c       deform.dat.  Note that where multiple entries occur for a single
c       state in om-deformations.dat, the code makes an ECIS96 input
c       for each entry.
c
      include "om-retrieve.cmb"
c
      character*3,ifin
      data isuit,ifin/0,'fin'/
      data kprnt1,kprnt2,kprnt3,kprnt4,kprnt5,kprnt6/0,0,0,0,0,0/
      data nee/21/
c
      open(unit=1,file='omparameter.dat')
      open(unit=5,file='ominput.inp')
      open(unit=35,file='omp-table')
c
c     Read input information
c
      nsc2=0
      read(5,*) ne
      if(ne.gt.0) go to 88
      nee=0
      go to 90
  88  read(5,*) (en(n),n=1,ne)
c
c     Statement 90 ==> Main Read-Cycle Return
c
  90  read(5,*,end=990) iztar,iatar,irefget,modtyp
      nonegw=0
      if(modtyp.lt.0) nonegw=1
      modtyp=iabs(modtyp)
      if(modtyp.eq.3)read(5,*)kdef
c
      irndp=0
      if(iztar.lt.0)irndp=1
      iztar=iabs(iztar)
      irndt=0
      if(iatar.lt.0)irndt=1
      iatar=iabs(iatar)
      izatar=1000*iztar+iatar
c
      if(modtyp.ne.1) go to 92
      kprnt1=1
      kprnt3=1
      go to 98
  92  if(modtyp.ne.2) go to 96
      if(imodel.ne.2) go to 94
      kprnt5=1
      go to 98
  94  kprnt2=1
      kprnt4=1
      go to 98
  96  kprnt6=1
c
c     Get optical model parameters
c
  98  rewind ki
 100  call omin11
      if(ieof.eq.1) go to 90
c
c     Set energy grid up to emax if nee=0
c
      if(nee.eq.0)call egrid(ne,emin,emax,en)
      call optname
c
c     Check for desired reference and Z,A range
 108  if(irefget.lt.0.and.imodel.lt.1) go to 110
      if(irefget.eq.iref) go to 112
      go to 100
 110  izapr=iabs(irefget)
      izpr=izapr/1000
      iapr=mod(izapr,1000)
      if(izpr.ne.izproj.or.iapr.ne.iaproj) go to 100
 112  if(izmin.eq.0.and.izmax.eq.0) go to 114
      if(iztar.lt.izmin.or.iztar.gt.izmax)
     +  write(6,'("Inputted target Z outside range of RIPL potential")')
 114  if(iamin.eq.0.and.iamax.eq.0) go to 120
      if(iatar.lt.iamin.or.iatar.gt.iamax)
     +  write(6,'("Inputted target A outside range of RIPL potential")')
c
c     Check if input file can be made for this potential
c
 120  if(imodel.le.2)go to 122
      write(6,'("imodel=",i2," in RIPL library not implemented yet.")')
      go to 90
c
 122  if(modtyp.ne.1)go to 124
      if(imodel.eq.0)go to 124
      write(6,'("Rotational or vibrational model inputs not implemented
     +for SCAT2000")')
      go to 90
c
 124  if(imodel.ne.1)go to 126
      if(idr.eq.0)go to 126
      write(6,'("Dispersive potentials for rotational model not implemen
     +ted.",/,"Non-dispersive rotational model input is substituted.")')
      go to 130
c
 126  if(imodel.eq.2.and.modtyp.eq.3)write(6,'("You are making an ECIS96
     + DWBA input from a vibrational potential.")')
c
      if(imodel.eq.1.and.modtyp.eq.3)write(6,'("You are making an ECIS96
     + DWBA input from a rotational potential.")')
c
c     Initialize program
 130  call setup
      kecis=0
c
c     Call output routines
      if(modtyp.ne.1)go to 150
c
c     Check to see if simple model used so that a block energy
c     array can be given to scatip2. If not, send one energy
c     at a time to scatip2
      nsc2=nsc2+1
      isimple=1
      if(nonegw.eq.1)go to 142
      do 136 i=1,6
      jab=iabs(jrange(i))
      if(jab.lt.1)go to 136
      do 134 j=1,jab
      if(pot(i,j,16).ne.0..or.pot(i,j,17).ne.0.)go to 142
      do 134 k=22,ndim3
      if(pot(i,j,k).ne.0.)go to 142
 134  continue
      if(jab.le.1)go to 136
      do 135 j=2,jab
      do 135 k=1,11
      if(rco(i,j,k).ne.rco(i,j-1,k))go to 142
      if(aco(i,j,k).ne.aco(i,j-1,k))go to 142
 135  continue
 136  continue
c
c     Send block energy array to scatip2
      nesc=ne
      do 140 n=1,ne
 140  ensc(n)=en(n)
      call scatip2
      go to 200
c
c     Send one energy at a time to scatip2
 142  isimple=2
      nesc=1
      do 146 n=1,ne
      ensc(1)=en(n)
      call scatip2
 146  continue
      go to 200
c
 150  continue
c
c     Check for Gaussian form factor for WD
      krange=iabs(jrange(4))
      if(krange.eq.0)go to 153
      do 151 j=1,krange
      if(rco(4,j,1).lt.0.)go to 152
 151  continue
      go to 153
 152  write(6,'("No ECIS96 inputs made when Gaussian form factor ",
     + "used for WD",/,"iref=",i5)')iref
c
 153  if(modtyp.ne.2)go to 160
c
c     Set up for table print
      call tableset
c
      if(imodel.eq.2)go to 154
      kecis=1
      call ecisip
      kecis=2
      call ecisip
      go to 200
c
c     vibrational model being used
 154  kecis=3
      call ecisdwip
      go to 200
 160  if(modtyp.ne.3) stop 5
c
c     DWBA model being used
      kecis=4
      call ecisdwip
c
 200  if(irefget.le.0) go to 100
      go to 90
c
c     End computation
 990  if(kprnt1.eq.0)go to 992
      backspace 4
      write(4,'(i5)')isuit
 992  if(kprnt2.eq.0)go to 994
      write(7,'(a3)') ifin
 994  if(kprnt3.eq.0)go to 996
      backspace 3
      write(3,'(i5)')isuit
 996  if(kprnt4.eq.0)go to 998
      write(9,'(a3)') ifin
 998  if(kprnt5.eq.0)go to 1000
      write(14,'(a3)') ifin
 1000 if(kprnt6.eq.0)go to 1020
      write(15,'(a3)') ifin
 1020 stop
      end
      subroutine omin11
c
c     routine to retrieve optical model parameters from RIPL-2 library
      include "om-retrieve.cmb"
c
  1   format(80a1)
c
c     Zero arrays
      call setr(0.,epot,5*ndim1)
      call setr(0.,rco,5*ndim1*ndim2)
      call setr(0.,aco,5*ndim1*ndim2)
      call setr(0.,pot,5*ndim1*ndim3)
c
      ieof=0
      read(ki,*,end=999) iref
      read(ki,1) (author(m),m=1,80)
      read(ki,1) (refer(m),m=1,80)
      read(ki,1) (summary(m),m=1,320)
      read(ki,*) emin,emax
      read(ki,*) izmin,izmax
      read(ki,*) iamin,iamax
      read(ki,*) imodel,izproj,iaproj,irel,idr
c
      do 100 m=1,6
        read(ki,*) jrange(m)
        if(jrange(m).eq.0) go to 100
        krange=iabs(jrange(m))
        do 98 j=1,krange
          read(ki,*) epot(m,j)
          read(ki,*) (rco(m,j,n),n=1,ndim2)
          read(ki,*) (aco(m,j,n),n=1,ndim2)
          read(ki,*) (pot(m,j,n),n=1,ndim3)
  98    continue
 100  continue
      read(ki,*) jcoul
      if(jcoul.le.0) go to 110
      do 108 j=1,jcoul
 108  read(ki,*) ecoul(j),rcoul0(j),rcoul(j),rcoul1(j),rcoul2(j),beta(j)
 110  if(imodel.ne.1) go to 130
      read(ki,*) nisotop
      do 120 n=1,nisotop
        read(ki,*) iz(n),ia(n),ncoll(n),lmax(n),idef(n),bandk(n),
     +    (def(n,k),k=2,idef(n),2)
        do 124 k=1,ncoll(n)
          read(ki,*) ex(k,n),spin(k,n),ipar(k,n)
 124    continue
 120  continue
      go to 200
 130  if(imodel.ne.2) go to 150
      read(ki,*) nisotop
      do 140 n=1,nisotop
        read(ki,*) iz(n),ia(n),nvib(n)
        do 138 k=1,nvib(n)
          read(ki,*) exv(k,n),spinv(k,n),iparv(k,n),nph(k,n),defv(k,n),
     +    thetm(k,n)
 138    continue
 140  continue
      go to 200
 150  if(imodel.ne.3)go to 200
      read(ki,*) nisotop
      do 160 n=1,nisotop
        read(ki,*) iz(n),ia(n),beta0(n),gamma0(n),xmubeta(n)
 160  continue
 200  continue
      read(ki,1,end=999)idum
      return
 999  ieof=1
      write(6,*)'End of RIPL-2 optical potential library reached.'
      return
      end
      subroutine setr (a,b,n)
c     ******************************************************************
c     set all elements of array b(n) to real number a.
c     ******************************************************************
      dimension b(n)
      do 100 k=1,n
  100 b(k)=a
      return
      end
      subroutine setup
c
c     Initialize program
      include "om-retrieve.cmb"
c
      becis1='FFFFFFFFFFFFFFFFFFFFFFFFFFFTFFFFFFFFFFFFFFFFFFFFFF'
      becis2='FFFFFFFFFFFFFTFFTTTFFTTFTFFFFFFFFFFFFFFFFFFFFFFFFF'
c
      ndisx=ndim8
      ifirst1=0
      ifirst2=0
      ifirst3=0
      ifirst4=0
      ifirst5=0
c
      izaproj=1000*izproj + iaproj
      if(izaproj.eq.1   ) ipsc=1
      if(izaproj.eq.1001) ipsc=2
      if(izaproj.eq.1002) ipsc=3
      if(izaproj.eq.1003) ipsc=4
      if(izaproj.eq.2003) ipsc=5
      if(izaproj.eq.2004) ipsc=6
      zproj=real(izproj)
      atar=float(iatar)
      ztar=float(iztar)
      eta=1.-(2.*ztar/atar)
      encoul=0.4*ztar/atar**(1./3.)
c
c     Get Fermi energy, masses and kinematic factor
      call masses
      if(irndt.eq.1) tarmas=atar
      if(irndp.eq.1) projmas=iaproj
c
c     Internal Flags
      koptom=2
c     Set koptom=1 to compute excited state potentials at incident
c         energy only. (Shortcut that is not recommended.)
c     Set koptom=2 to compute excited state potentials at incident
c         energy minus the excitation energy (corrected to lab).
c         This is the recommended option.
c
      return
      end
      subroutine scatip2
c
c     routine to write optical model parameters in parameter lib
      include "om-retrieve.cmb"
c
      parameter(npcofx=7)
      dimension rsc(6),resc(6),asc(6),aesc(6),potsc(6,ndim1,npcofx),
     + npzen(6),epotsc(6,ndim1),b(6,ndim1,12)
c
c======= Capote 2001 =======
c
c     To use dispersive optical model package
c
      real*8 Delta_WD,Delta_WV
      real*8 DOM_int,DOM_int_T1,DOM_int_T2
      real*8 EEE,As,Bs,Cs,AAv,Bv,Ep,EFF,Ea,Ef
      real*8 WVE,WDE,Ecut
      real*8 DWS,DWV,WV2,WV4,WD2,WD4
c
      common /energy/EEE,Ef,Ep
      common /Wenerg/WDE,WVE
      common /pdatas/As,Bs,Cs
      common /pdatav/AAv,Bv
c
      external Delta_WD,Delta_WV,WV2,WV4,WD2,WD4
c======= Capote 2001 =======
c
c
      data isuit/1/
      data ipr,ida,iba,ipu/1,1,1,1/
c     data ipr,ida,iba,ipu/1,0,0,1/
c
      if(ifirst1.ne.25)open(unit=4,file='sc2.inp')
      ifirst1=25
      ko=4
c
      call setr(0.,rsc,6)
      call setr(0.,resc,6)
      call setr(0.,asc,6)
      call setr(0.,aesc,6)
      call setr(0.,potsc,6*npcofx*ndim1)
c
c     Get coulomb radius, if needed
      el=ensc(1)
      rcoulsc=0.
      if(jcoul.lt.1)go to 100
      jc=1
      if(isimple.ne.2)go to 90
      do 80 j=1,jcoul
      if(el.gt.ecoul(j))jc=j+1
  80  continue
      jc=min0(jc,jcoul)
  90  rcoulsc=rcoul(jc) + rcoul0(jc)*atar**(-1./3.) +
     +       rcoul1(jc)*atar**(-2./3.) + rcoul2(jc)*atar**(-5./3.)
      betasc=beta(jc)
 100  encoul2=0.
      if(rcoulsc.gt.0.) encoul2=1.73*ztar/(rcoulsc*atar**(1./3.))
      ipot=0
      do 150 i=1,6
      jab=iabs(jrange(i))
      ii=i
      if(i.eq.2)ii=3
      if(i.eq.3)ii=2
      npzen(ii)=jrange(i)
      if(jrange(i).eq.0)go to 150
      do 104 j=1,jab
 104  epotsc(ii,j)=epot(i,j)
c
c     Check to see if simple or complex potential
      if(isimple.eq.2)go to 110
c
c     Standard SCAT2 potential formulas
      j=1
      rsc(ii)=abs(rco(i,j,1)) + rco(i,j,3)*eta
     *       + rco(i,j,4)/atar + rco(i,j,5)/sqrt(atar)
     *       + rco(i,j,6)*atar**(2./3.) + rco(i,j,7)*atar
     *       + rco(i,j,8)*atar**2  + rco(i,j,9)*atar**3
     *       + rco(i,j,10)*atar**(1./3.)
     *       + rco(i,j,11)*atar**(-1./3.)
      resc(ii)=rco(i,j,2)
      asc(ii)=abs(aco(i,j,1)) + aco(i,j,3)*eta
     *       + aco(i,j,4)/atar + aco(i,j,5)/sqrt(atar)
     *       + aco(i,j,6)*atar**(2./3.) + aco(i,j,7)*atar
     *       + aco(i,j,8)*atar**2 + aco(i,j,9)*atar**3
     *       + aco(i,j,10)*atar**(1./3.) + aco(i,j,11)*atar**(-1./3.)
      aesc(ii)=aco(i,j,2)
      do 108 j=1,jab
      potsc(ii,j,1)=pot(i,j,1) + eta*pot(i,j,7) + encoul*pot(i,j,8)
     *             + pot(i,j,9)*atar + pot(i,j,10)*atar**(1./3.)
     *             + pot(i,j,11)*atar**(-2./3.) + pot(i,j,12)*encoul2
      potsc(ii,j,2)=pot(i,j,2) + pot(i,j,13)*eta + pot(i,j,14)*atar
      potsc(ii,j,3)=pot(i,j,3)
      potsc(ii,j,4)=pot(i,j,4)
      potsc(ii,j,5)=pot(i,j,5) + pot(i,j,15)*eta
      potsc(ii,j,6)=pot(i,j,6)
 108  continue
      go to 150
c
c     Find energy range that energy el falls in.
 110  jp=1
      do 112 j=1,jab
      if(el.gt.epot(i,j))jp=j+1
 112  continue
      j=min0(jp,jab)
      js=1
      epotsc(ii,js)=epot(i,j)
      if(npzen(ii).gt.0)npzen(ii)=+1
      if(npzen(ii).lt.0)npzen(ii)=-1
c
      Ef=dble(int(100000*efermi))/100000
      if(pot(i,j,18).ne.0.) Ef=pot(i,j,18) + pot(i,j,19)*atar
c
c     Calculate radius and diffuseness parameters
      rsc(ii)=abs(rco(i,j,1)) + rco(i,j,3)*eta
     *       + rco(i,j,4)/atar + rco(i,j,5)/sqrt(atar)
     *       + rco(i,j,6)*atar**(2./3.) + rco(i,j,7)*atar
     *       + rco(i,j,8)*atar**2  + rco(i,j,9)*atar**3
     *       + rco(i,j,10)*atar**(1./3.)
     *       + rco(i,j,11)*atar**(-1./3.)
      resc(ii)=rco(i,j,2)
      asc(ii)=abs(aco(i,j,1)) + aco(i,j,3)*eta
     *       + aco(i,j,4)/atar + aco(i,j,5)/sqrt(atar)
     *       + aco(i,j,6)*atar**(2./3.) + aco(i,j,7)*atar
     *       + aco(i,j,8)*atar**2 + aco(i,j,9)*atar**3
     *       + aco(i,j,10)*atar**(1./3.) + aco(i,j,11)*atar**(-1./3.)
      aesc(ii)=aco(i,j,2)
c
      if(pot(i,j,24).eq.0.) go to 120
c
c     Special Koning-type potential formulas
c
c======= Capote 2001 =======
c
c     Retrieving energy above which nonlocality in the volume absorptive
c               potential is considered (Ea)
      Ea=dble(int(100000*pot(i,j,21)))/100000
      if(Ea.eq.0.) Ea=1000.1d0
      Ep=dble(int(100000*pot(i,j,20)))/100000
      if(Ep.eq.0.) Ep=Ef
c
      eff=Ef
c     Retrieving average energy of the particle states Ep
      if( (i.eq.2 .OR. i.eq.4) .and. (Ep.ne.Ef) ) eff=Ep
c
      if(i.eq.1) call bcoget(b,j)
c
      elf=el-eff
      vc=0.
      if(i.gt.1.or.b(1,j,5).eq.0.)go to 116
      vc=b(1,j,1)*encoul2*(b(1,j,2)-2.*b(1,j,3)*elf+3.*b(1,j,4)*elf**2)
 116  nn = int(pot(i,j,13))
c
      potsc(ii,js,1)=
     +    b(i,j,1)*(1.- b(i,j,2)*elf+b(i,j,3)*elf**2 -
     +    b(i,j,4)*elf**3) + b(i,j,5)*vc +
     +    b(i,j,6)*(elf**nn/(elf**nn + b(i,j,7)**nn)) +
     +    b(i,j,8)*exp(-b(i,j,9)*elf)*(elf**nn/
     +    (elf**nn + b(i,j,10)**nn)) +
     +    b(i,j,11)*exp(-b(i,j,12)*elf)
c
c     Nonlocality consideration
      if(i.eq.2 .and. Ea.lt.1000. .and. el.gt.(Ef+Ea) )
     +         potsc(ii,js,1) = potsc(ii,js,1) +
     +       1.65*(sqrt(el)+(Ef+Ea)**1.5/(2.*el)-1.5*sqrt(Ef+Ea))
c
c======= Capote 2001 =======
c
      go to 150
c
 120  if(pot(i,j,23).eq.0)go to 130
c
c     Special Varner-type potential formulas
      potsc(ii,js,1)= (pot(i,j,1) + pot(i,j,2)*eta)/
     *    (1.+ exp((pot(i,j,3) - el + pot(i,j,4)*encoul2)/pot(i,j,5)))
      if(pot(i,j,6).eq.0.)go to 150
      potsc(ii,js,1)= potsc(ii,js,1)
     *    + pot(i,j,6)*exp((pot(i,j,7)*el - pot(i,j,8))/pot(i,j,6))
      go to 150
c
 130  if(pot(i,j,22).eq.0)go to 140
c
c     Special Smith-type potential formulas
      pi=acos(-1.d0)
      potsc(ii,js,1)=pot(i,j,1) + pot(i,j,2)*eta
     *    + pot(i,j,6)*exp(pot(i,j,7)*el + pot(i,j,8)*el*el)
     *    + pot(i,j,9)*el*exp(pot(i,j,10)*el**pot(i,j,11))
      if(pot(i,j,5).ne.0.)potsc(ii,js,1)=potsc(ii,js,1)
     *    + pot(i,j,3)*cos(2.*pi*(atar - pot(i,j,4))/pot(i,j,5))
      go to 150
c
c     Standard potential formulas
 140  potsc(ii,js,1)=pot(i,j,1) + pot(i,j,7)*eta + pot(i,j,8)*encoul
     *    + pot(i,j,9)*atar + pot(i,j,10)*atar**(1./3.)
     *    + pot(i,j,11)*atar**(-2./3.) + pot(i,j,12)*encoul2
     *    + (pot(i,j,2) + pot(i,j,13)*eta + pot(i,j,14)*atar)*el
     *    + pot(i,j,3)*el*el + pot(i,j,4)*el*el*el + pot(i,j,6)*sqrt(el)
     *    + (pot(i,j,5) + pot(i,j,15)*eta + pot(i,j,16)*el)*alog(el)
     *    + pot(i,j,17)*encoul/el**2
c
 150  continue
c
      if(rco(4,1,1).lt.0.) rsc(4)=-rsc(4)
      if(nonegw.eq.0) go to 152
      if(potsc(3,js,1).lt.0.0) potsc(3,js,1)=0.
      if(potsc(4,js,1).lt.0.0) potsc(4,js,1)=0.
c
c======= Capote 2001 =======
c
c     To calculate dispersion relation contribution
 152  if(idr.eq.2 .and. (pot(2,1,24).ne.0 .or. pot(4,1,24).ne.0) ) then
        EEE=el
c       Exact calculation of the dispersive contribution
        i=2
c       Only one energy range
        j=1
c       Real volume contribution from Dispersive relation
        dwv=0.d0
c
        pi=acos(-1.d0)
c       RCN 03/2003
c       Ecut=1000.
        Ecut=150000.
c
        AAv=dble(int(100000*pot(i,j,1)))/100000
        Bv =dble(int(100000*pot(i,j,3)))/100000

        if(pot(i,j,13).eq.2) THEN
c           DOM integral
            WVE=WV2(AAv,Bv,Ep,EEE)
c           RCN 03/2003
c           DWV=2*DOM_int(Delta_WV,WV2,Ef,Ef+50.d0,Ecut,EEE,WVE)
            DWV=2*DOM_int(Delta_WV,WV2,Ef,Ef+5.*Bv,Ecut,EEE,0.d0)
c           Nonlocality correction to the DOM integral (only used if Ea =
c           is defined)
            if(Ea.lt.1000.)
     +        DWV=DWV+DOM_int_T1(AAv,Ef,Ea,EEE)+DOM_int_T2(Ef,Ea,EEE)
        endif

        if(pot(i,j,13).eq.4) THEN
c           DOM integral
            WVE=WV4(AAv,Bv,Ep,EEE)
c           RCN 03/2003
c           DWV=2*DOM_int(Delta_WV,WV4,Ef,Ef+50.d0,Ecut,EEE,WVE)
            DWV=2*DOM_int(Delta_WV,WV4,Ef,Ef+5.*Bv,Ecut,EEE,0.d0)
c           Nonlocality correction to the DOM integral (only used if Ea =
c           is defined)
            if(Ea.lt.1000.)
     +        DWV=DWV+DOM_int_T1(AAv,Ef,Ea,EEE)+DOM_int_T2(Ef,Ea,EEE)
c
        endif
c
        i=4
c       Only one energy range
        j=1
c       Real volume contribution from Dispersive relation
        dws=0.d0
        As=dble(int(100000*pot(i,j,1)))/100000
        Bs=dble(int(100000*pot(i,j,6)))/100000
        Cs=dble(int(100000*pot(i,j,2)))/100000
c
c       RCN 03/2003
        Ecut=2000.
c
        if(pot(i,j,13).eq.2) THEN
c           DOM integral
            WDE=WD2(As,Bs,Cs,Ep,EEE)
c           RCN 03/2003
c           dws = 2*DOM_int(Delta_WD,WD2,Ef,Ef+20.d0,Ecut,EEE,WDE)
            dws = 2*DOM_int(Delta_WD,WD2,Ef,Ef+30.d0,Ecut,EEE,WDE)
        endif

        if(pot(i,j,13).eq.4) THEN
c           DOM integral
            WDE=WD4(As,Bs,Cs,Ep,EEE)
            dws = 2*DOM_int(Delta_WD,WD4,Ef,Ef+30.d0,Ecut,EEE,WDE)
        endif
c       Adding real volume dispersive contribution to the real potential
c       Geometry parameters are the same as for the volume
c       potential(imag and real)
        potsc(1,1,1) = potsc(1,1,1)+dwv
c       Including real surface dispersive contribution
c       Geometry parameters are the same as for the imaginary surface
c       potential
        potsc(2,1,1)=dws
        rsc(2)=rsc(4)
        asc(2)=asc(4)
        npzen(2)=1
      endif
c
c     Write SCAT2 input
c
      if(nsc2.gt.1)go to 156
      if(isimple.gt.1.or.jcoul.le.1)go to 154
      write(6,*)'First energy range used for coulomb radius'
      write(ko,*)'First energy range used for coulomb radius'
c154  write(ko,'(6i5)') ipr,ida,iba,ipu,idr,irel
 154  itmp=irel
      if(irel.eq.2) itmp=1
c
c     Dispersion relations are considered explicitly in this code
c     so SCAT2 does not need to make any DR related calculation
c         => we are setting idr = 0 always
      write(ko,'(6i5)') ipr,ida,iba,ipu, 0 ,itmp
      nsc2=nsc2+1
 156  write(ko,'(i5)') nesc
      ensc(1)=-ensc(1)
      write(ko,'(7f10.5)') (ensc(j),j=1,nesc)
      ensc(1)=-ensc(1)
      write(ko,'(2i5)') iztar,iatar
      write(ko,'(2i5)') ipsc,ipot
c
c======= Capote 2001 =======
c
c     Factor coming from Dirac equation reduction (relativistic
c     approximation).
c     Using this factor force us to employ relativistic kinematics as
c     well.
      gamma=1.d0
      if(irel.eq.2) then
        amu=931.5012d0
c       Target system mass in MeV
        EMtar=tarmas*amu
c       Total system mass in MeV
        EMtot=(tarmas+projmas)*amu
c       Total kinetic energy in cm
        Tcm=sqrt(2*EMtar*el + EMtot**2) - EMtot
c       Relativistic correction to the potential (non relativistic
c       target!!).
        gamma=1.d0+Tcm/(Tcm+2*projmas*amu)
c       GAMMA NOT NEEDED FOR SCAT2000 SO SET GAMMA=1.
        gamma=1.d0
      endif
c
c======= Capote 2001 =======
c
      do 200 i=1,6
      write(ko,'(4f9.5,i5)') rsc(i),resc(i),asc(i),aesc(i),npzen(i)
c     if(npzen(i).eq.0)go to 200
      do 160 j=1,iabs(npzen(i))
      write(ko,5)epotsc(i,j),(gamma*potsc(i,j,k),k=1,npcofx)
  5   format(f9.3,1p,4(1x,e12.5),/,9x,3(1x,e12.5))
 160  continue
 200  continue
      write(ko,'(3f10.5)') rcoulsc,efermi,betasc
      write(ko,'(i5)') isuit
c     close (unit=ko)
      return
      end
      subroutine ecisip
c
c     ROUTINE TO PREPARE AND WRITE ECIS96 INPUT
      include "om-retrieve.cmb"
c
      if(kecis.eq.2) go to 40
      if(ifirst2.ne.25)open(unit=7,file='ec96.inp')
      ifirst2=25
      ko=7
      go to 42
  40  if(ifirst4.ne.25)open(unit=9,file='ec96tc.inp')
      ifirst4=25
      ko=9
c
c
  42  astep=1.
      ncol=1
      iterm=1
      npp=1
      rmatch=0.
      convg=1.0e-8
      ecis1=becis1
      ecis2=becis2
      ecis2(15:15)='T'
      ecis2(20:20)='T'
      if(imodel.eq.0)ecis2(40:40)='T'
      if (izaproj.ne.1) ecis2(10:10)='T'
c     if (irel.eq.1) ecis1(8:8)='T'
      if (irel.ge.1) ecis1(8:8)='T'
      if(imodel.ne.1)go to 20
      call couch11
      ncol=1
      if(ntar.gt.0)ncol=ncoll(ntar)
      npp=ncol
      if(koptom.eq.1)npp=1
  20  if(kecis.eq.1) go to 50
      ecis2(13:13)='T'
      ecis2(14:14)='F'
      ecis2(15:15)='F'
      astep=180.
  50  do 90 n=1,ne
      el=en(n)
      if(kecis.eq.2)el=en(n)/xkine
c
c     Check for gaussian ff for WD
      call gaussff(igauflg)
      if(igauflg.eq.1)go to 90
c
      if(imodel.ne.1.or.ntar.eq.0)go to 54
      if(spin(1,ntar).ge.2.5.and.el.lt.0.1)convg=1.0e-10
  54  ldwmax=2.4*1.25*iatar**(1./3.)*0.22*sqrt(projmas*en(n))
c     njmax=max(2*ldwmax,20)
      njmax=max(ldwmax,20)
      jchk=1
      if(kecis.ne.1)jchk=0
      call optmod
      call optname
      write(ko,'(f7.3," MeV ",a8," on ",i3,a2," - Optical model: ",
     * 12a1," REF#=",i5)') el,parname(ipsc),iatar,nuc(iztar),opname,iref
      write(ko,'(a50)') ecis1
      write(ko,'(a50)') ecis2
      write(ko,'(4i5)') ncol,njmax,iterm,npp
      write(ko,'(10x,f10.5,10x,1p,3(2x,e8.1))') rmatch,convg,convg,convg
      if (ecis2(15:15).eq.'T') write(ko,'( )')
      if(imodel.ne.1)go to 60
      call couch22
      go to 62
  60  tgts=0.
      tgtp='+'
      write(ko,'(f5.2,2i2,a1,5f10.5)') tgts,0,1,tgtp,el,
     +  projspi,projmas,tarmas,ztar*zproj
      write(ko,'( )')
  62  do 70 j=1,npp
      if(npp.eq.1)go to 68
      el=en(n)
      if(kecis.eq.2)el=el/xkine
      if(ntar.gt.0)el=el-ex(j,ntar)/xkine
      if(el.lt.0.) el=0.0001
      jchk=0
      if(j.gt.1)call optmod
  68  continue
      write(ko,'(3f10.5)') v,rv,av
      write(ko,'(3f10.5)') w,rw,aw
      write(ko,'(3f10.5)') vd,rvd,avd
      write(ko,'(3f10.5)') wd,rwd,awd
      write(ko,'(3f10.5)') vso,rvso,avso
      write(ko,'(3f10.5)') wso,rwso,awso
      write(ko,'(3f10.5)') rc,0.,0.
      write(ko,'(3f10.5)') 0.,0.,0.
      if(j.eq.npp)write(ko,'(3f10.5)') 0.,astep,180.
  70  continue
  90  continue
c     close (unit=ko)
      return
      end
      subroutine ecisdwip
c
c     Routine to prepare and write ECIS96 inputs for vibrational and
c       DWBA cases.
c
      include "om-retrieve.cmb"
      character*4 lmodel
      character*1 tgtpx
c
      if(modtyp.eq.3) go to 12
      if(ifirst6.ne.25)open(unit=15,file='ec96vib.inp')
      ifirst6=25
      ko=15
      go to 14
  12  if(ifirst5.ne.25)open(unit=14,file='ec96dw.inp')
      ifirst5=25
      ko=14
c
  14  ncol=2
      iterm=0
      npp=2
        if(koptom.eq.1)npp=1
      rmatch=0.
      convg=1.0e-8
      ecis1=becis1
      ecis2=becis2
      ecis1(12:12)='T'
      if (izaproj.ne.1) ecis2(10:10)='T'
c     if (irel.eq.1) ecis1(8:8)='T'
      if (irel.ge.1) ecis1(8:8)='T'
c
      if(modtyp.eq.2) call vib1
      kexit=0
      if(modtyp.eq.3) call dwba1(kexit)
      if(kexit.eq.1)return
      lmodel='VIBR'
      if(modtyp.eq.3)lmodel='DWBA'
c
      ecis2(14:14)='F'
      ecis2(15:15)='F'
      ecis2(30:30)='T'
      tgts=sdis(1)
      tgtp='+'
      if(tgts.eq.0.0.and.ipdis(1).ge.0)go to 60
      write(6,'("Nucleus izatar=",i6,": VIB and DWBA options only",
     +  " set up for even-even targets.")') izatar
      stop 16
  60  do 90 i=1,ne
        eninc=en(i)
        ecm=xkine*eninc
c
c       Check for gaussian ff for WD
        el=eninc
        call gaussff(igauflg)
        if(igauflg.eq.1)go to 90
c
c       if(tgts.ge.2.5.and.eninc.lt.0.1) convg=1.0e-10
        ldwmax=2.4*1.25*iatar**(1./3.)*0.22*sqrt(projmas*eninc)
c       njmax=max(2*ldwmax,20)
        njmax=max(ldwmax,20)
        do 88 j=2,ndis
          if (ecm.le.edis(j)+0.1) go to 88
          tgtpx='+'
          if(ipdis(j).lt.0)tgtpx='-'
        write(ko,'(f6.2," MeV ",a8," on ",$)') eninc,parname(ipsc)
        write(ko,'(i3,a2,": ",a4," for ",$)')iatar,nuc(iztar),lmodel
        write(ko,'(f4.1,a1," level at ",f5.3," MeV")')
     +   sdis(j),tgtpx,edis(j)
        write(ko,'(a50)') ecis1
        write(ko,'(a50)') ecis2
        write(ko,'(4i5)') ncol,njmax,iterm,npp
        write(ko,'(10x,f10.5,10x,1p,3(2x,e8.1))') rmatch,convg,convg,
     + convg
        if (ecis2(15:15).eq.'T') write(ko,'( )')
        write(ko,'(f5.2,2i2,a1,5f10.5)') tgts,0,1,tgtp,eninc,
     +    projspi,projmas,tarmas,ztar*zproj
        write(ko,'(2i5)') 0,0
          jj=2
          if(koptom.eq.1)jj=1
c         if(typdis(j).eq.'D')bdis(j)=bdis(j)/(rv*atar**(1./3.))
          write(ko,'(f5.2,2i2,a1,5f10.5)') sdis(j),0,jj,tgtpx,edis(j),
     +      projspi,projmas,tarmas,ztar*zproj
          write(ko,'(2i5)') 1,1
          xjdis=abs(sdis(1)-sdis(j)) + 0.001
          jdis=int(xjdis)
          if(modtyp.eq.3)jdis=langmo(j)
          write(ko,'(i5,5x,f10.5)') jdis,bdis(j)
        do 80 k=1,npp
          if (k.gt.1)go to 74
          el=eninc
          jchk=0
          go to 76
   74     el=eninc-edis(j)/xkine
          if(el.lt.0.) el=0.0001
   76     call optmod
          write(ko,'(3f10.5)') v,rv,av
          write(ko,'(3f10.5)') w,rw,aw
          write(ko,'(3f10.5)') vd,rvd,avd
          write(ko,'(3f10.5)') wd,rwd,awd
          write(ko,'(3f10.5)') vso,rvso,avso
          write(ko,'(3f10.5)') wso,rwso,awso
          write(ko,'(3f10.5)') rc,0.,0.
          write(ko,'(3f10.5)') 0.,0.,0.
   80   continue
        write(ko,'(3f10.5)') 0.,180.,180.
   88 continue
   90 continue
      write(ko,'("fin")')
c     close (unit=ko)
      return
      end
      subroutine vib1
c
c     Routine to setup for vibrational model calculation
c
      include "om-retrieve.cmb"
c
      ecis2(20:20)='T'
c
      ntar=0
      do 44 nis=1,nisotop
      if(iz(nis).ne.iztar.or.ia(nis).ne.iatar)go to 44
      ntar=nis
      go to 48
  44  continue
      write(6,'("Nucleus izatar=",i6," does not have vibrational",
     +  " data.")') izatar
      stop 6
c
  48  ndis=nvib(ntar)
      do 54 k=1,ndis
      edis(k)=exv(k,ntar)
      sdis(k)=spinv(k,ntar)
      ipdis(k)=iparv(k,ntar)
  54  bdis(k)=defv(k,ntar)
      return
      end
      subroutine dwba1(kexit)
c
c     Routine to setup for vibrational model calculation
c
      include "om-retrieve.cmb"
      character*2 sym,idummy
      character*10 authz,audef1,audef2
c
      if(kdef.eq.1) audef1='JENDL-3.2 '
      if(kdef.eq.1) audef2='JENDL-3.2 '
      if(kdef.eq.2) audef1='ENSDF(Q)  '
      if(kdef.eq.2) audef2='ENSDF(Q)  '
      if(kdef.eq.3) audef1='ENSDF(BE2)'
      if(kdef.eq.3) audef2='ENSDF(BE3)'
      if(kdef.eq.4) audef1='Raman     '
      if(kdef.eq.4) audef2='Spear     '
      if(kdef.eq.5) audef1='User      '
      if(kdef.eq.5) audef2='User      '
c
      ecis2(20:20)='T'
      if(modtyp.eq.3)ecis2(42:42)='T'
c
      open(unit=16,file='deform.dat')
c
      klev=1
      edis(klev)=0.
      sdis(klev)=0.
      ipdis(klev)=0
      langmo(klev)=0
      bdis(klev)=0.
c
c     Position deform.dat file
      do 25 i=1,4
  25  read(16,'(a2)') idummy
c
  30  read(16,'(2i4,1x,a2,1x,f10.6,1x,f4.1,i3,i2,1x,f10.6,2x,a10)',
     + end=900) kz,ka,sym,edisz,sdisz,ipdisz,langmoz,bdisz,authz
      kza=1000*kz+ka
      if(kza.gt.izatar)go to 900
      if(kza.ne.izatar)go to 30
      if(kdef.eq.5)go to 40
      if(authz.ne.audef1.and.authz.ne.audef2) go to 30
c
  40  klev=klev+1
      edis(klev)=edisz
      sdis(klev)=sdisz
      ipdis(klev)=ipdisz
      langmo(klev)=langmoz
      bdis(klev)=bdisz
      go to 30
c
 900  ntar=0
      ndis=klev
      if(ndis.eq.1) go to 990
      ndis=min0(ndisx,ndis)
      go to 999
c
 990  write(6,'("DWBA deformations not found for izatar=",i7)') izatar
      kexit=1
 999  close (unit=16)
      return
      end
      subroutine gaussff(igauflg)
c
c     Routine to see if Gaussian form factor required for WD(el)
c
      include "om-retrieve.cmb"
c
      igauflg=0
      jab=iabs(jrange(4))
      if(jrange(4).lt.1)go to 40
c
      jp=1
      do 30 j=1,jab
      if(el.gt.epot(4,j)) jp=j+1
  30  continue
      j=min0(jp,jab)
      if(rco(4,j,1).lt.0.)igauflg=1
c
  40  return
      end
      subroutine optmod
c
c     Routine to generate input for ECIS96 from RIPL-2 library
c
      include "om-retrieve.cmb"
      dimension rlib(6),alib(6),vlib(6),b(6,ndim1,12)
c
c======= Capote 2001 =======
c
c     To use dispersive optical model package
c
      real*8 Delta_WD,Delta_WV
      real*8 DOM_int,DOM_int_T1,DOM_int_T2
      real*8 EEE,As,Bs,Cs,AAv,Bv,Ep,EFF,Ea,Ef
      real*8 WVE,WDE,Ecut
      real*8 DWS,DWV,WV2,WV4,WD2,WD4
c
      common /energy/EEE,Ef,Ep
      common /Wenerg/WDE,WVE
      common /pdatas/As,Bs,Cs
      common /pdatav/AAv,Bv
c
      external Delta_WD,Delta_WV,WV2,WV4,WD2,WD4
c
c======= Capote 2001 =======
c
c     Generate optical model parameters for ECIS
c
      pi=acos(-1.d0)
      rc=0.
      if(jcoul.lt.1)go to 194
      jc=1
      do 190 j=1,jcoul
      if(el.gt.ecoul(j)) jc=j+1
 190  continue
      jc=min0(jc,jcoul)
      rc=rcoul0(jc)*atar**(-1./3.) + rcoul(jc) +
     +   rcoul1(jc)*atar**(-2./3.) + rcoul2(jc)*atar**(-5./3.)
 194  encoul2=0.
      if(rc.gt.0.) encoul2=1.73*ztar/(rc*atar**(1./3.))
c
      do 300 i=1,6
      rlib(i)=0.
      alib(i)=0.
      vlib(i)=0.
      jab=iabs(jrange(i))
      if(jrange(i).lt.1)go to 300
      jp=1
      do 204 j=1,jab
      if(el.gt.epot(i,j)) jp=j+1
 204  continue
      j=min0(jp,jab)
      Ef=dble(int(100000*efermi))/100000
      if(pot(i,j,18).ne.0.) Ef=pot(i,j,18) + pot(i,j,19)*atar
c
      rlib(i)=abs(rco(i,j,1)) + rco(i,j,2)*el + rco(i,j,3)*eta
     *        + rco(i,j,4)/atar + rco(i,j,5)/sqrt(atar)
     *        + rco(i,j,6)*atar**(2./3.) + rco(i,j,7)*atar
     *        + rco(i,j,8)*atar**2  + rco(i,j,9)*atar**3
     *        + rco(i,j,10)*atar**(1./3.)
     *        + rco(i,j,11)*atar**(-1./3.)
c
      alib(i)=abs(aco(i,j,1)) + aco(i,j,2)*el + aco(i,j,3)*eta
     *        + aco(i,j,4)/atar + aco(i,j,5)/sqrt(atar)
     *        + aco(i,j,6)*atar**(2./3.) + aco(i,j,7)*atar
     *        + aco(i,j,8)*atar**2 + aco(i,j,9)*atar**3
     *        + aco(i,j,10)*atar**(1./3.) + aco(i,j,11)*atar**(-1./3.)
c
c
      if (pot(i,j,24).eq.0.) go to 210
c
c     Special Koning-type potential formulas
c
c======= Capote 2001 =======
c
c     Retrieving energy above which nonlocality in the volume absorptive
c               potential is considered (Ea)
      Ea=dble(int(100000*pot(i,j,21)))/100000
      if(Ea.eq.0.) Ea=1000.1d0
      Ep=dble(int(100000*pot(i,j,20)))/100000
      if(Ep.eq.0.) Ep=Ef
c
      eff=Ef
c     Retrieving average energy of the particle states Ep
      if( (i.eq.2 .or. i.eq.4) .and. (Ep.ne.Ef) ) eff=Ep
c
      if(i.eq.1) call bcoget(b,j)
c
      elf=el-eff
      vc=0.
      if(i.gt.1.or.b(1,j,5).eq.0.)go to 206
      vc=b(1,j,1)*encoul2*(b(1,j,2)-2.*b(1,j,3)*elf+3.*b(1,j,4)*elf**2)
 206  nn = int(pot(i,j,13))
c
       vlib(i)=
     +    b(i,j,1)*(1.- b(i,j,2)*elf + b(i,j,3)*elf**2 -
     +    b(i,j,4)*elf**3) + b(i,j,5)*vc +
     +    b(i,j,6)*(elf**nn/(elf**nn + b(i,j,7)**nn)) +
     +    b(i,j,8)*exp(-b(i,j,9)*elf)*(elf**nn/
     +    (elf**nn + b(i,j,10)**nn)) +
     +    b(i,j,11)*exp(-b(i,j,12)*elf)
c
c     Nonlocality consideration
      if(i.eq.2 .and. Ea.lt.1000. .and. el.gt.(Ef+Ea) )
     +         vlib(i)= vlib(i) +
     +       1.65*(sqrt(el)+(Ef+Ea)**1.5/(2.*el)-1.5*sqrt(Ef+Ea))
c
c======= Capote 2001 =======
c
      go to 300
c
 210  if (pot(i,j,23).eq.0.) go to 220
c
c     Special Varner-type potential formulas
      vlib(i)= (pot(i,j,1) + pot(i,j,2)*eta)/
     *    (1.+ exp((pot(i,j,3) - el + pot(i,j,4)*encoul2)/pot(i,j,5)))
      if(pot(i,j,6).eq.0.)go to 300
      vlib(i) = vlib(i)
     *    + pot(i,j,6)*exp((pot(i,j,7)*el - pot(i,j,8))/pot(i,j,6))
      go to 300
c
 220  if (pot(i,j,22).eq.0.) go to 230
c
c     Special Smith-type potential formulas
      vlib(i)=pot(i,j,1) + pot(i,j,2)*eta
     *    + pot(i,j,6)*exp(pot(i,j,7)*el + pot(i,j,8)*el*el)
     *    + pot(i,j,9)*el*exp(pot(i,j,10)*el**pot(i,j,11))
      if(pot(i,j,5).ne.0.)vlib(i)=vlib(i)
     *    + pot(i,j,3)*cos(2.*pi*(atar - pot(i,j,4))/pot(i,j,5))
      go to 300
c
c     Standard potential formulas
 230  vlib(i)=pot(i,j,1) + pot(i,j,7)*eta + pot(i,j,8)*encoul
     *    + pot(i,j,9)*atar + pot(i,j,10)*atar**(1./3.)
     *    + pot(i,j,11)*atar**(-2./3.) + pot(i,j,12)*encoul2
     *    + (pot(i,j,2) + pot(i,j,13)*eta + pot(i,j,14)*atar)*el
     *    + pot(i,j,3)*el*el + pot(i,j,4)*el*el*el + pot(i,j,6)*sqrt(el)
      if(el.gt.0.) vlib(i) = vlib(i) + pot(i,j,17)*encoul/el**2
     *    + (pot(i,j,5) + pot(i,j,15)*eta + pot(i,j,16)*el)*alog(el)
 300  continue
c
c======= Capote 2001 =======
c
c     To calculate dispersion relation contribution
      if(idr.eq.2 .and. (pot(2,1,24).ne.0 .or. pot(4,1,24).ne.0) ) then
        EEE=el
c       Exact calculation of the dispersive contribution
        i=2
c       Only one energy range
        j=1
c       Real volume contribution from Dispersive relation
        dwv=0.d0

        pi=acos(-1.d0)
c       RCN 03/2003
c       Ecut=1000.
        Ecut=150000.

        AAv=dble(int(100000*pot(i,j,1)))/100000
        Bv =dble(int(100000*pot(i,j,3)))/100000

        if(pot(i,j,13).eq.2) THEN
c           DOM integral
            WVE=WV2(AAv,Bv,Ep,EEE)
c           RCN 03/2003
c           DWV=2*DOM_int(Delta_WV,WV2,Ef,Ef+50.d0,Ecut,EEE,WVE)
            DWV=2*DOM_int(Delta_WV,WV2,Ef,Ef+5.*Bv,Ecut,EEE,0.d0)
c           Nonlocality correction to the DOM integral (only used if Ea
c           is defined).
            if(Ea.lt.1000.)
     +        DWV=DWV+DOM_int_T1(AAv,Ef,Ea,EEE)+DOM_int_T2(Ef,Ea,EEE)
c
        endif
c
        if(pot(i,j,13).eq.4) THEN
c           DOM integral
            WVE=WV4(AAv,Bv,Ep,EEE)
c           RCN 03/2003
c           DWV=2*DOM_int(Delta_WV,WV4,Ef,Ef+50.d0,Ecut,EEE,WVE)
            DWV=2*DOM_int(Delta_WV,WV4,Ef,Ef+5.*Bv,Ecut,EEE,0.d0)
c           Nonlocality correction to the DOM integral (only used if Ea
c           is defined).
            if(Ea.lt.1000.)
     +        DWV=DWV+DOM_int_T1(AAv,Ef,Ea,EEE)+DOM_int_T2(Ef,Ea,EEE)
c
        endif

        i=4
c       Only one energy range
        j=1
c       Real volume contribution from Dispersive relation
        dws=0.d0
        As=dble(int(100000*pot(i,j,1)))/100000
        Bs=dble(int(100000*pot(i,j,6)))/100000
        Cs=dble(int(100000*pot(i,j,2)))/100000
c
c       RCN 03/2003
        Ecut=2000.
c
        if(pot(i,j,13).eq.2) THEN
c           DOM integral
            WDE=WD2(As,Bs,Cs,Ep,EEE)
c           RCN 03/2003
c           dws = 2*DOM_int(Delta_WD,WD2,Ef,Ef+20.d0,Ecut,EEE,WDE)
            dws = 2*DOM_int(Delta_WD,WD2,Ef,Ef+30.d0,Ecut,EEE,WDE)
        endif

        if(pot(i,j,13).eq.4) THEN
c           DOM integral
            WDE=WD4(As,Bs,Cs,Ep,EEE)
            dws = 2*DOM_int(Delta_WD,WD4,Ef,Ef+30.d0,Ecut,EEE,WDE)
        endif
c       Adding real volume dispersive contribution to the real potential
c       Geometry parameters are the same as for the volume
c       potential(imag and real).
        vlib(1)=vlib(1)+dwv
c       Including real surface dispersive contribution
c       Geometry parameters are the same as for the imaginary surface
c       potential.
        vlib(3)=dws
        alib(3)=alib(4)
        rlib(3)=rlib(4)

      endif
c
c======= Capote 2001 =======
c
c     Factor coming from Dirac equation reduction (relativistic
c     approximation)
c     Using this factor forces us to employ relativistic kinematics as
c     well.
      gamma=1.d0
      if(irel.eq.2) then
        amu=931.5012d0
c       Target system mass in MeV
        EMtar=tarmas*amu
c       Total system mass in MeV
        EMtot=(tarmas+projmas)*amu
c       Total kinetic energy in cm
        Tcm=sqrt(2*EMtar*el + EMtot**2) - EMtot
c       Relativistic correction to the potential (non relativistic
c       target)
        gamma=1.d0+Tcm/(Tcm+2*projmas*amu)
c       GAMMA NOT NEEDED FOR ECIS96 SO SET GAMMA=1.
        gamma=1.d0
      endif
c======= Capote 2001 =======
c
      v=vlib(1)*gamma
      rv=rlib(1)
      av=alib(1)
      w =vlib(2)*gamma
      if(w.lt.0.0.and.nonegw.eq.1) w=0.
      rw=rlib(2)
      aw=alib(2)
      vd=vlib(3)*gamma
      rvd=rlib(3)
      avd=alib(3)
      wd=vlib(4)*gamma
      if(wd.lt.0.0.and.nonegw.eq.1) wd=0.
      rwd=rlib(4)
      awd=alib(4)
      vso=vlib(5)*gamma
      rvso=rlib(5)
      avso=alib(5)
      wso=vlib(6)*gamma
      rwso=rlib(6)
      awso=alib(6)
      if(jchk.eq.1)write(35,'(f7.3,6(f7.3,f6.3,f6.3),f6.3)') el,v,rv,
     + av,w,rw,aw,vd,rvd,avd,wd,rwd,awd,vso,rvso,avso,wso,rwso,awso
      return
      end
      subroutine bcoget(b,j)
c
c     Routine to compute b coefficients for Koning global potential
c
      include "om-retrieve.cmb"
      dimension b(6,ndim1,12)
c
      call setr(0.,b,72*ndim1)
c
       b(1,j,1)  =  pot(1,j,1) + pot(1,j,2)*atar + pot(1,j,8)*eta
       b(1,j,2)  =  pot(1,j,3) + pot(1,j,4)*atar
       b(1,j,3)  =  pot(1,j,5) + pot(1,j,6)*atar
       b(1,j,4)  =  pot(1,j,7)
       b(1,j,5)  =  pot(1,j,9)
       b(1,j,11) =  pot(1,j,10) + pot(1,j,11)*atar
       b(1,j,12) =  pot(1,j,12)
       b(2,j,6)  =  pot(2,j,1) + pot(2,j,2)*atar
       b(2,j,7)  =  pot(2,j,3) + pot(2,j,4)*atar
       b(4,j,8)  =  pot(4,j,1) + pot(4,j,8)*eta
       if(pot(4,j,3).ne.0.) then
         b(4,j,9)  =  pot(4,j,2) +
     +              pot(4,j,3)/(1. + exp((atar-pot(4,j,4))/pot(4,j,5)))
       else
         b(4,j,9)  =  pot(4,j,2)
       endif
       b(4,j,10) =  pot(4,j,6)
       b(5,j,11) =  pot(5,j,10) + pot(5,j,11)*atar
       b(5,j,12) =  pot(5,j,12)
       b(6,j,6)  =  pot(6,j,1)
       b(6,j,7)  =  pot(6,j,3)
c
      return
      end
      subroutine optname
c
c     Routine to load opname with first author name (or first
c     12 characters).
c
      include "om-retrieve.cmb"
      character*1 opnamex
      dimension opnamex(12)
c
      do 20 i=1,12
  20  opnamex(i)=author(i)
      do 22 i=2,12
      j=i
      if(opnamex(i).eq.' '.and.opnamex(i-1).ne.'.')go to 24
      if(opnamex(i).eq.',')go to 24
  22  continue
  24  do 26 ii=1,j
      i=j-ii+1
      if(opnamex(i).ne.' '.and.opnamex(i).ne.',')go to 28
  26  continue
  28  jj=i
      do 30 i=1,12
  30  opname(i)=' '
      do 32 i=1,jj
  32  opname(i)=opnamex(i)
c
c     Set names for table print
      if(imodel.eq.0)modelz='spherical  '
      if(imodel.eq.1)modelz='rotational '
      if(imodel.eq.2)modelz='vibrational'
      if(imodel.eq.3)modelz='non-axial  '
      if(idr.eq.0)dispz='non-dispersive'
      if(idr.gt.0)dispz='dispersive    '
      if(irel.eq.0)relz='non-relativistic'
      if(irel.gt.0)relz='relativistic    '
      return
      end
      subroutine couch11
c
c     First setup for coupled channels parameters
c
      include "om-retrieve.cmb"
c
      ecis1(1:1)='T'
      ecis1(12:12)='T'
      ecis1(21:21)='T'
      ntar=0
      do 44 nis=1,nisotop
      if(iz(nis).ne.iztar.or.ia(nis).ne.iatar)go to 44
      ntar=nis
      go to 48
  44  continue
      write(6,'("Nucleus izatar=",i6," does not have coupled-channel",
     +  " data.")') izatar
      write(6,'("RIPL-2 reference number: ",i6)')iref
c     stop 6
  48  return
      end
      subroutine couch22
c
c     Second setup for coupled channels parameters
c
      include "om-retrieve.cmb"
c
      if(ntar.ne.0)go to 54
      write(ko,'("PUT ROTATIONAL BAND STATES IN HERE")')
      go to 62
  54  do 56 k=1,ncol
      elx=ex(k,ntar)
      if(k.eq.1) elx=el
      tarspi=spin(k,ntar)
      tarpar='+'
      if(ipar(k,ntar).eq.-1)tarpar='-'
      kom=k
      if(koptom.eq.1)kom=1
      write(ko,'(f5.2,2i2,a1,5f10.5)') tarspi,0,kom,tarpar,elx,
     +  projspi,projmas,tarmas,ztar*zproj
  56  continue
      iqm=idef(ntar)
      iqmax=lmax(ntar)
      aspin=bandk(ntar)
      ik=1
      write(ko,'(2i5,f10.5,i5)') iqm,iqmax,aspin,ik
c     iq=iqm/2
      write(ko,'(7f10.5)') (def(ntar,j),j=2,iqm,2)
  62  return
      end
      subroutine masses
c
c     Routine to retrieve masses and compute separation energies
c     and Fermi energy
c
      include "om-retrieve.cmb"
      character*1 dp
c
c     Compute lab to cm factor
      zatar=float(izatar)
      call energy2(zatar,tarmas,tarspi,tarpar,tarex)
      zaproj=float(izaproj)
      call energy2(zaproj,projmas,projspi,projpar,projex)
      xkine=tarmas/(projmas+tarmas)
c
c     Compute Fermi energy
      call energy2(zatar-zaproj,dm,ds,dp,tarexm1)
      call energy2(zatar+zaproj,dm,ds,dp,tarexp1)
      efermi=-0.5*(tarexm1-tarexp1+2.*projex)
      return
      end
      subroutine energy2(za,exactmas,spin,parity,excess)
c
c     Routine energy2 looks up values of ground-state mass excess
c     (in MeV), spin, and parity. Missing data produces a flag.
c     Note that negative za omits Duflo for missing data.
c
      common/xener/nmass,inpgrd,lza(9200),ener(9200),spnpar(9200)
      character*4 bcd
      character*1 parity
      dimension bcd(120)
c
      data amu,k13/931.5012,13/
c
  1   format(28h0***** ground-state data for,i6,19h not in table *****)
  5   format(5x,'+++++ ground state of ',f6.0,' is incompletely describe
     +d,     spin,parity = ',f6.2,a1,2x,'+++++')
  6   format(5x,'+++++',28x,'assignments changed to,   spin,parity =
     +',f6.2,a1,2x,'+++++')
  8   format(///' M A S S   D A T A   I N P U T   T O   S U B R O U T I
     +N E   E N E R G Y',/(20a4))
c
c     FIRST CALL CAUSES DATA TO BE READ IN FROM UNIT 13
      if(inpgrd.eq.12345) go to 10
      open(unit=13,status='unknown',file='gs-mass-sp.dat')
      open(unit=44,status='unknown',file='massinfo.out')
      read(k13,'(20a4)')bcd
      write(44,8)bcd
      read(k13,'(i7)')nmass
      read(k13,'(5(i7,f11.6,f7.1))')(lza(n),ener(n),spnpar(n),n=1,nmass)
      rewind k13
      inpgrd = 12345
      close (unit=k13)
c
c     SET FLAG FOR DUFLO OPTION
 10   imiss=1
      if(za) 12,15,20
 12   za=abs(za)
      imiss=0
      go to 20
c
c     Z=0,A=0 IS CONSIDERED A PHOTON
   15 excess=0.
      spin=0.
      parity='-'
      exactmas=0.
      return
c
c     FIND REQUESTED NUCLEUS IN TABLE
   20 iza=ifix(za)
      ia=mod(iza,1000)
      amass=float(ia)
      if(iza.lt.lza(1).or.iza.gt.lza(nmass))go to 40
      in=1
      if(iza.eq.lza(1))go to 50
      il=1
      iu=nmass
 122  ii=(il+iu)/2
      if(lza(ii)-iza)124,130,125
 124  il=ii
      go to 126
 125  iu=ii
 126  if(iu-il-1)135,140,122
 130  in=ii
      go to 50
 135  in=il
      go to 150
 140  in=iu
 150  if(lza(in).ne.iza)go to 40
      go to 50
c
c     REQUESTED ISOTOPE IS NOT IN TABLES
 40   write(44,1) iza
      excess= -999999.
      if(imiss.eq.0) stop 7776
      npro=iza/1000
      nneu=ia-npro
      call duflo(nneu,npro,excess)
      exactmas=amass+excess/amu
      go to 200
c
c     PREPARE RETRIEVED DATA FOR EXIT
 50   continue
      excess=ener(in)
      exactmas=amass+excess/amu
      if(spnpar(in).ge.9900.)spin=spnpar(in)-9900.
      if(spnpar(in).ge.9900.)parity='u'
      if(spnpar(in).ge.9900.)go to 200
      if(spnpar(in).ge.100.)parity='+'
      if(spnpar(in).ge.100.)spin=spnpar(in)-100.
      if(spnpar(in).lt.0.)parity='-'
      if(spnpar(in).lt.0.)spin=spnpar(in)+100.
 200  kspin=spin+0.01
      if((parity.ne.'u').and.(kspin.ne.99))return
      write(44,5) za,spin,parity
      if(parity.eq.'u')parity='+'
      if(kspin.eq.99)spin=.25*(1.-(-1.)**ia)
      write(44,6) spin,parity
      return
      end
      subroutine duflo(nn,nz,excess)
c
c                --------------------------------------------
c                |Nuclear mass formula of Duflo-Zuker (1992)|
c                --------------------------------------------
c
      dimension xmag(6),zmag(5),a(21)
      data zmag/14.,28.,50.,82.,114./
      data xmag/14.,28.,50.,82.,126.,184./
      data a/16.178,18.422,120.146,202.305,12.454,0.73598,5.204,1.0645,
     +1.4206,0.0548,0.1786,.6181,.0988,.0265,-.1537,.3113,-.6650,-.0553,
     +-.0401,.1774,.4523/
c
      x=nn
      z=nz
      t=abs(x-z)*.5
      v=x+z
      s=v**(2./3.)
      u=v**(1./3.)
c     a5=a(5)
c     if(z.gt.x) a5=0.
      E0=a(1)*v-a(2)*s-a(3)*t*t/v+a(4)*t*t/u/v-a(5)*t/u-a(6)*z*z/u
      esh=0.
      esh1=0.
      do 10 k=2,5
        f1=zmag(k-1)
        f2=zmag(k)
        dfz=f2-f1
        if(z.ge.f1.and.z.lt.f2) then
          roz=(z-f1)/dfz
          pz=roz*(roz-1)*dfz
          do 20 l=2,6
            f3=xmag(l-1)
            f4=xmag(l)
            dfn=f4-f3
            if(x.ge.f3.and.x.lt.f4) then
              ron=(x-f3)/dfn
              pn=ron*(ron-1)*dfn
              esh=(pn+pz)*a(8)+a(10)*pn*pz
              xx=2.*ron-1.
              zz=2.*roz-1.
              txxx=pn*xx
              tzzz=pz*zz
              txxz=pn*zz
              tzzx=pz*xx
              kl=l-k
              if(kl.eq.0) esh1=a(k+10)*(txxx+tzzz)+a(k+15)*(txxz+tzzx)
              if(kl.eq.1)
     +          esh1=a(k+11)*txxx-a(k+16)*txxz+a(k+10)*tzzz-a(k+15)*tzzx
              if(kl.eq.2)
     +          esh1=a(k+12)*txxx+a(k+17)*txxz+a(k+10)*tzzz+a(k+15)*tzzx
                edef=a(9)*(pn+pz)+a(11)*pn*pz
                if(esh.lt.edef) esh=edef
            end if
   20     continue
        end if
   10 continue
      ebin=e0+esh+esh1
      nn2=nn/2
      nz2=nz/2
      nn2=2*nn2
      nz2=2*nz2
      if(nn2.ne.nn) ebin=ebin-a(7)/u
      if(nz2.ne.nz) ebin=ebin-a(7)/u
      amu=931.5012
      rneumas=1.008665
      rpromas=1.007825
      amass=nz*rpromas+nn*rneumas-ebin/amu
      excess=(amass-nz-nn)*amu
      return
      end
      subroutine egrid(ne,eminx,emaxx,en)
c     Routine to generate auto energy grid.
c     Currently setup to produce grid for A. Koning
c
      dimension en(500),enx(500)
c
      enx(1)=0.
      enx(2)=0.001
      enx(3)=0.002
c
c     energy range 0-0.010 MeV, delta E = 0.002 MeV
      do 10 i=4,7
  10  enx(i)=enx(i-1)+0.002
c
c    energy range 0.01-0.10 MeV, delta E = 0.005 MeV
      do 20 i=8,25
  20  enx(i)=enx(i-1)+0.005
c
c    energy range 0.1-1.0 MeV, delta E = 0.05 MeV
      do 30 i=26,43
  30  enx(i)=enx(i-1)+0.05
c
c    energy range 1.0-10.0 MeV, delta E = 0.2 MeV
      do 40 i=44,88
  40  enx(i)=enx(i-1)+0.2
c
c    energy range 10-200 MeV, delta E = 0.5 MeV
      do 50 i=89,468
      nex=i
  50  enx(i)=enx(i-1)+0.5
c
c     Now put energy grid between potential limits.
      ne=1
      do 80 n=2,nex
      if(enx(n).lt.eminx.or.enx(n).gt.emaxx)go to 80
      ne=ne+1
      en(ne)=enx(n)
  80  continue
      return
      end
      subroutine tableset
c
c     Routine to setup for table print
c
      include "om-retrieve.cmb"
c
      write(35,'(/"Reference Number =",i5," Incident Particle: ",a8)')
     +  iref,parname(ipsc)
      write(35,'("Target: Z=",i3," A=",i4," Number of energies=",i4)')
     +     iztar,iatar,ne
      write(35,'("Type potential: ",a11,2x,a14,2x,a16)')
     +              modelz,dispz,relz
      write(35,'("First Author: ",12a1)') (opname(i),i=1,12)
      write(35,'("Z-Range=",i2,"-",i3,"  A-Range=",i3,"-",i3,"  E-Range="
     +",f8.3,"-",f8.3," MeV")')izmin,izmax,iamin,iamax,emin,emax
      write(35,'(/,"En(MeV)     V    RV    AV      W    RW    AW ",
     +                    "    VD   RVD   AVD     WD   RWD   AWD ",
     +                    "   VSO   RVSO  AVSO   WSO   RWSO  AWSO ")')
      return
      end
      block data
      include "om-retrieve.cmb"
c     integer ki,idr
c     common/blk1/ki,idr,en(300),parname(7),nuc(103)
      data ki,idr/1,0/
      data (nuc(i),i=1,103) /'H ','He','Li','Be','B ','C ','N ','O ',
     +  'F ','Ne','Na','Mg','Al','Si','P ','S ','Cl','Ar','K ','Ca',
     +  'Sc','Ti','V ','Cr','Mn','Fe','Co','Ni','Cu','Zn','Ga','Ge',
     +  'As','Se','Br','Kr','Rb','Sr','Y ','Zr','Nb','Mo','Tc','Ru',
     +  'Rh','Pd','Ag','Cd','In','Sn','Sb','Te','I ','Xe','Cs','Ba',
     +  'La','Ce','Pr','Nd','Pm','Sm','Eu','Gd','Tb','Dy','Ho','Er',
     +  'Tm','Yb','Lu','Hf','Ta','W ','Re','Os','Ir','Pt','Au','Hg',
     +  'Tl','Pb','Bi','Po','At','Rn','Fr','Ra','Ac','Th','Pa','U ',
     +  'Np','Pu','Am','Cm','Bk','Cf','Es','Fm','Md','No','Lr'/
      data (parname(i),i=1,7) /'neutron ','proton  ','deuteron',
     +  'triton  ','he-3    ','alpha   ','gamma   '/
c     end block data
      end
C
C==========================================================================
C     AUTHOR: Dr. Roberto Capote
C
C     e-mail: rcapotenoy@yahoo.com
C
C     DISPERSIVE OPTICAL MODEL POTENTIAL PACKAGE
C
      REAL*8 FUNCTION DOM_int_T1(Av,Ef,Ea,E)
C
C     Integral over E' corresponding to nonlocal additions T1(E'<<0)
C
c     IMPLICIT REAL*8(A-H,O-Z)
      real*8 E,Ea,Ef,Av,Ex,Ea2,Ea3,Ea4,Eax
C
      Pi=acos(-1.d0)
C
      Ex=E-Ef
      Ea2=Ea**2
      Ea3=Ea2*Ea
      Ea4=Ea2*Ea2
      Eax=Ex+Ea
C
      DDEN1 = 2*Pi*Ea2*(Eax**2+Ea2)
      DNUM1 = 0.5d0*Pi*Ex*(Ea3-Ea2*Eax) - Ea2*Ex*(Ex+2*Ea)*dlog(Ea)
     >   + Ea2*Eax**2*dlog(dabs(Eax/Ea)) + Ea2*Eax**2*dlog(dabs(Eax))
     >   - Ea4*dlog(Ea)
C
      DOM_int_T1 = -Av*DNUM1/DDEN1
C
      RETURN
      END
C
      REAL*8 FUNCTION DOM_int_T2(Ef,Ea,E)
C
C     Integral over E' corresponding to nonlocal additions T2(E'>>0)
C
c     IMPLICIT REAL*8(A-H,O-Z).
      real*8 E,Ea,Ef,Pi,El,Alpha,R1,R2,R3,R4
        Pi=acos(-1.d0)
        El=Ef+Ea
        Alpha = 1.65d0
c
        R1=1.5*DSQRT(El)*dLOG(abs((El-E)/Ea))
c
        IF(E.eq.0.d0) THEN
          R2=0.5*El**1.5d0*(1.d0/El-dlog(abs(El/Ea))/Ef)
        ELSE
          R2=0.5*El**1.5d0/(E*Ef)*
     >             ( Ef*dlog(dabs(El/(El-E))) -E*dlog(dabs(El/Ea)) )
        ENDIF
c
        R3=2*DSQRT(dABS(Ef))*( 0.5d0*Pi - atan( DSQRT(El/dabs(Ef)) ) )
c
        IF(E.GE.0.d0) THEN
          R4=DSQRT(E)*
     >          dlog(dabs( (dsqrt(El)+dsqrt(E))/(dsqrt(El)-dsqrt(E)) ))
        ELSE
C         R4=-2.d0*DSQRT(dabs(E))*( 0.5d0*Pi - atan( DSQRT(dabs(El/E)) ) )
        ENDIF
c
        DOM_int_T2 =  Alpha/Pi*(R1+R2+R3+R4)
c
      RETURN
      END
C
      real*8 function DELTA_WV(WV,y)
      real*8 E,Ef,A,B,Ep,y,WV,WDE,WVE
      common /energy/E,Ef,Ep
      common /Wenerg/WDE,WVE
      common /pdatav/A,B
C
      DELTA_WV=(WV(A,B,Ep,y) - WVE)
     >         /((y-Ef)**2-(E-Ef)**2)
      return
      end
C
      real*8 function WV4(A,B,Ep,E)
      real*8 A,B,Ep,E
C
      WV4=0.d0
      if(E.LE.Ep) return
C
      ee=(E-Ep)**4
      WV4=A*ee/(ee+B**4)
C
      return
      end
C
      real*8 function WV2(A,B,Ep,E)
      real*8 A,B,Ep,E
C
      WV2=0.d0
      if(E.LE.Ep) return
C
      ee=(E-Ep)**2
      WV2=A*ee/(ee+B**2)
C
      return
      end
C
      real*8 function DELTA_WD(WD,y)
      real*8 E,Ef,A,B,C,Ep,y,WD,WDE,WVE
      common /energy/E,Ef,Ep
      common /Wenerg/WDE,WVE
      common /pdatas/A,B,C
C
      DELTA_WD=(WD(A,B,C,Ep,y) - WDE)
     >            /((y-Ef)**2-(E-Ef)**2)
C
C     DELTA_WD=(WD(A,B,C,Ep,y) -
C    >          WD(A,B,C,Ep,E))
C    >          /((y-Ef)**2-(E-Ef)**2)
C
      return
      end
C
      real*8 function WD4(A,B,C,Ep,E)
      real*8 A,B,C,Ep,E,ee,arg
C
      WD4=0.d0
      if(E.Lt.Ep) return
      arg=C*(E-Ep)
      IF(arg.GT.15) return
      ee=(E-Ep)**4
      WD4=A*ee/(ee+B**4)*EXP(-arg)
      return
      end
C
      real*8 function WD2(A,B,C,Ep,E)
      real*8 A,B,C,Ep,E,ee,arg
C
      WD2=0.d0
      if(E.Lt.Ep) return
      arg=C*(E-Ep)
      IF(arg.GT.15) return
      ee=(E-Ep)**2
      WD2=A*ee/(ee+B**2)*EXP(-arg)
      return
      end
C
      REAL*8 FUNCTION DOM_int(DELTAF,F,Ef,Eint,Ecut,E,WE_cte)
C
C     DOM integral (20 points Gauss-Legendre)
C
C     Divided in two intervals for higher accuracy
C     The first interval corresponds to peak of the integrand
C
      DOUBLE PRECISION Eint,Ef,Ecut,WE_cte,E
      DOUBLE PRECISION F,WG,XG,WWW,XXX,DELTAF
      DOUBLE PRECISION ABSC1,CENTR1,HLGTH1,RESG1
      DOUBLE PRECISION ABSC2,CENTR2,HLGTH2,RESG2
      INTEGER J
      EXTERNAL F
      DIMENSION XG(10),WG(10)
C
C     THE ABSCISSAE AND WEIGHTS ARE GIVEN FOR THE INTERVAL (-1,1).
C     BECAUSE OF SYMMETRY ONLY THE POSITIVE ABSCISSAE AND THEIR
C     CORRESPONDING WEIGHTS ARE GIVEN.
C
C     XG - ABSCISSAE OF THE 41-POINT GAUSS-KRONROD RULE
C     WG - WEIGHTS OF THE 20-POINT GAUSS RULE
C
C GAUSS QUADRATURE WEIGHTS AND KRONROD QUADRATURE ABSCISSAE AND WEIGHTS
C AS EVALUATED WITH 80 DECIMAL DIGIT ARITHMETIC BY L. W. FULLERTON,
C BELL LABS, NOV. 1981.
C
      DATA WG  (  1) / 0.0176140071 3915211831 1861962351 853 D0 /
      DATA WG  (  2) / 0.0406014298 0038694133 1039952274 932 D0 /
      DATA WG  (  3) / 0.0626720483 3410906356 9506535187 042 D0 /
      DATA WG  (  4) / 0.0832767415 7670474872 4758143222 046 D0 /
      DATA WG  (  5) / 0.1019301198 1724043503 6750135480 350 D0 /
      DATA WG  (  6) / 0.1181945319 6151841731 2377377711 382 D0 /
      DATA WG  (  7) / 0.1316886384 4917662689 8494499748 163 D0 /
      DATA WG  (  8) / 0.1420961093 1838205132 9298325067 165 D0 /
      DATA WG  (  9) / 0.1491729864 7260374678 7828737001 969 D0 /
      DATA WG  ( 10) / 0.1527533871 3072585069 8084331955 098 D0 /
C
      DATA XG( 1) / 0.9931285991 8509492478 6122388471 320 D0 /
      DATA XG( 2) / 0.9639719272 7791379126 7666131197 277 D0 /
      DATA XG( 3) / 0.9122344282 5132590586 7752441203 298 D0 /
      DATA XG( 4) / 0.8391169718 2221882339 4529061701 521 D0 /
      DATA XG( 5) / 0.7463319064 6015079261 4305070355 642 D0 /
      DATA XG( 6) / 0.6360536807 2651502545 2836696226 286 D0 /
      DATA XG( 7) / 0.5108670019 5082709800 4364050955 251 D0 /
      DATA XG( 8) / 0.3737060887 1541956067 2548177024 927 D0 /
      DATA XG( 9) / 0.2277858511 4164507808 0496195368 575 D0 /
      DATA XG(10) / 0.0765265211 3349733375 4640409398 838 D0 /
C
      CENTR1 = 0.5D+00*(Ef+Eint)
      HLGTH1 = 0.5D+00*(Eint-Ef)
      CENTR2 = 0.5D+00*(Ecut+Eint)
      HLGTH2 = 0.5D+00*(Ecut-Eint)
C
C     COMPUTE THE 20-POINT GAUSS-KRONROD APPROXIMATION
C     TO THE INTEGRAL in TWO INTERVALS (Ef - Eint, Eint - Ecut)
C
      RESG1 = 0.0D+00
      RESG2 = 0.0D+00
      DO J=1,10
      XXX=XG(J)
      WWW=WG(J)
        ABSC1 = HLGTH1*XXX
      RESG1=RESG1 + WWW*(DELTAF(F,CENTR1-ABSC1)+DELTAF(F,CENTR1+ABSC1))
        ABSC2 = HLGTH2*XXX
      RESG2=RESG2 + WWW*(DELTAF(F,CENTR2-ABSC2)+DELTAF(F,CENTR2+ABSC2))
      ENDDO
c
      CORR=0.5d0*WE_cte/(E-Ef)*dlog((Ecut-(E-Ef))/(Ecut+(E-Ef)))
      DOM_int = ( RESG1*HLGTH1+RESG2*HLGTH2 + CORR) *
     >           (E-Ef)/(acos(-1.d0))
c
      RETURN
      END
